//  Copyright © 2012 Stephen Smally
//      This program is free software; you can redistribute it and/or modify
//      it under the terms of the GNU General Public License as published by
//      the Free Software Foundation; either version 2 of the License, or
//      (at your option) any later version.
//      
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//      
//      You should have received a copy of the GNU General Public License
//      along with this program; if not, write to the Free Software
//      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
//      MA 02110-1301, USA.
//      

using PackageKit;

namespace Lsc.Backend {
    public enum LoadingType {
        PACKAGES, // Loaded packages
        CATEGORIES, // Loaded categories
        OTHER; // Installed or so
    }
    
    public class AppsManager : Object {
        // Signals
        public signal void app_added (Package app);
        public signal void category_added (string category, string group);
        public signal void loading_started (string text);
        public signal void loading_finished (LoadingType load);
        public signal void details_received (Package pkg, Details details);
        
        // PackageKit stuffs
        private Client client;
        private Control control;
        private Cancellable transaction;
        
        // Vars
        public string[] icon_table;
        
        public void get_pkgs_cb (Progress progress, ProgressType type) {
        }
        
        public void get_pkgs (string filters) {
            Bitfield filter;
            filter = Filter.bitfield_from_string(filters);
            loading_started("Loading packages");
            
            client.get_packages_async.begin(filter, null, (p) => {
                if (p.package != null) {
                    stdout.printf("Get package: %s\n", p.package.get_name());
                    app_added(p.package);
                }
            }, () => {
                loading_finished(LoadingType.PACKAGES);
            });
        }
        
        public void search_for_packages (string filters, string name) {
            Bitfield filter;
            filter = Filter.bitfield_from_string(filters);
            loading_started("Loading packages");
            client.search_names_async(filter, {name}, null, (pkg) => {
                if (pkg.package != null) {
                    app_added(pkg.package);
                }
                while (Gtk.events_pending())
                    Gtk.main_iteration();
            }, () => {
                loading_finished(LoadingType.PACKAGES);
            });
        }
        
        public void get_details (string pkg_id) {
            client.get_details_async({pkg_id},
            null,
            () => {},
            (obj, res) => {
                Results contains = client.generic_finish(res);
                details_received(get_package_from_id(pkg_id), contains.get_details_array()[0]);
            });
        }
        
        public Package get_package_from_id (string pkg_id) {
            Results res = client.resolve(Filter.bitfield_from_string(""),
            {pkg_id},
            null,
            null);
            return res.get_package_array()[0];
        }
        
        private void fill_icon_table () {
        }
        
        public void get_categories () {
            /* Need a wrapper for /usr/share/app-install/desktop/applications.menu*/
            
            category_added("applications-accessories", "Accessories");
            category_added("applications-internet", "Internet");
            category_added("applications-office", "Office");
            category_added("applications-multimedia", "Multimedia");
            category_added("applications-development", "Development");
            category_added("category-show-all", "All");
            
            loading_finished(LoadingType.CATEGORIES);
        }
        
        public void get_screenshot (string pkg_name) {
            
        } 
        
        public AppsManager () {
            client = new Client();
            control = new Control();
            transaction = new Cancellable();
            fill_icon_table();
        }
    }
}
