//
//  Copyright © 2012 Stephen Smally
//      This program is free software; you can redistribute it and/or modify
//      it under the terms of the GNU General Public License as published by
//      the Free Software Foundation; either version 2 of the License, or
//      (at your option) any later version.
//      
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//      
//      You should have received a copy of the GNU General Public License
//      along with this program; if not, write to the Free Software
//      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
//      MA 02110-1301, USA.
//      

using Gtk;
using Cairo;

namespace Lsc.Widgets {
    public class RoundBox : EventBox {
        private Cairo.Context cr;
        private Box container;
        
        private enum Colors {
            BACKGROUND,
            BORDER
        }
        
        public RoundBox (Orientation or, int spacing) {
            set_has_window(false);
            
            container = new Box(or, spacing);
            
            add(container);
        }
        
        public void pack_start (Widget widget, bool expand, bool fill, int space) {
            container.pack_start(widget, expand, fill, space);
        }
        
        private void draw_rounded_rectangle (Colors rgb, uint x, uint y, int width, int height, double radius) {
            Gdk.RGBA color;
            bool res;
            if (rgb == Colors.BACKGROUND)
                res = get_style_context().lookup_color("base_color", out color);
            else
                color = { 0.8, 0.8, 0.8, 1 };
                res = true;
            
            if (res == false) {
                color = { 0, 0, 0, 0 };
            }
            
            Gdk.cairo_set_source_rgba (cr, color);
            
            cr.new_path();
            cr.move_to (x + radius, y);
            cr.arc (width - x - radius, y + radius, radius, Math.PI * 1.5, Math.PI * 2);
            cr.arc (width - x - radius, height - y - radius, radius, 0, Math.PI * 0.5);
            cr.arc (x + radius, height - y - radius, radius, Math.PI * 0.5, Math.PI);
            cr.arc (x + radius, y + radius, radius, Math.PI, Math.PI * 1.5);
            cr.close_path ();
            
            cr.fill();
        }
        
        public override bool draw (Cairo.Context cr) {
            this.cr = cr;
            
            int width = get_allocated_width ();
            int height = get_allocated_height ();
            double arc_radius = 10.0;
            
            draw_rounded_rectangle(Colors.BORDER, 0, 0, width, height, arc_radius);
            draw_rounded_rectangle(Colors.BACKGROUND, 1, 1, width, height, arc_radius-1);
            
            return base.draw(cr);
        }
    
        public override void size_allocate (Allocation allocation) {
            base.size_allocate (allocation);
        }
    }
    
    public class  GridView : RoundBox {
        // Widgets
        public Box container;
        private Separator separator;
        private Label title_label;
        private Image hinter;
        
        // Vars
        public string title {
            set {
                title_label.label = "<b>"+value.replace("&", "&amp;")+"</b>";
            }
        }
        
        public string hint {
            set {
                if (value == "") {
                    hinter.set_visible(false);
                } else {
                    hinter.set_visible(true);
                    hinter.tooltip_text = value;
                }
            }
        }
        
        public void grid_pack_start (Widget widget, bool expand, bool fill, int space) {
            container.pack_start(widget, expand, fill, space);
        }
        
        private void pack_separator () {
            separator = new Separator(Orientation.HORIZONTAL);
            separator.margin_left = 1;
            separator.margin_right = 1;
            
            pack_start(separator, false, false, 0);
        }
        
        public GridView (string title) {
            base(Orientation.VERTICAL, 0);
            
            container = new Box(Orientation.VERTICAL, 5);
            container.border_width = 5;
            
            Box title_box = new Box(Orientation.HORIZONTAL, 5);
            title_box.border_width = 5;
            
            title_label = new Label("");
            title_label.use_markup = true;
            title_label.halign = Align.START;
            
            hinter = new Image.from_stock(Stock.HELP, IconSize.MENU);
            hinter.no_show_all = true;
            
            title_box.pack_start(title_label, false, false, 0);
            title_box.pack_end(hinter, false, false, 0);
            
            this.title = title;
            hint = "";
            
            pack_start(title_box, false, false, 0);
            pack_separator();
            pack_start(container, false, false, 0);
        }
    }
}
