//  Copyright © 2012 Stephen Smally
//      This program is free software; you can redistribute it and/or modify
//      it under the terms of the GNU General Public License as published by
//      the Free Software Foundation; either version 2 of the License, or
//      (at your option) any later version.
//      
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//      
//      You should have received a copy of the GNU General Public License
//      along with this program; if not, write to the Free Software
//      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
//      MA 02110-1301, USA.
//      

using Gtk;
using PackageKit;
using Lsc.Widgets;
using Lsc.Utils;

namespace Lsc.Pages {
    public class AppsInfo : Box {
        private Separator separator;
        private RoundBox box;
        public RoundBox reviews_box;
        private Label pkg_name;
        private Label short_description;
        private Label description;
        private Label id;
        private Label version;
        private Label size;
        
        public void set_details (Package pkg, Details details) {
            pkg_name.label = "<big><b>"+nicer_pkg_name(pkg.get_name())+"</b></big>";
            short_description.label = "<i>"+pkg.get_summary()+"</i>";
            
            description.label = details.description;
            
            id.label = "<b>Id:</b> "+pkg.get_name();
            version.label = "<b>Version:</b> "+pkg.get_version();
            size.label = "<b>Size:</b> "+details.size.to_string();
        }
        
        public void pack_separator () {
            separator = new Separator(Orientation.HORIZONTAL);
            separator.margin_left = 1;
            separator.margin_right = 1;
            box.pack_start(separator, false, false, 0);
        }
        
        public AppsInfo () {
            orientation = Orientation.VERTICAL;
            spacing = 5;
            border_width = 5;
            
            // Main container
            box = new RoundBox(Orientation.VERTICAL, 0);
            box.expand = false;
            
            // Summary Box (icon, name/short_description and install button)
            Box summary_box = new Box(Orientation.HORIZONTAL, 3);
            Box label_box = new Box(Orientation.VERTICAL, 0);
            label_box.valign = Align.CENTER;
            summary_box.border_width = 5;
            summary_box.pack_start(new Image.from_stock(Stock.INFO, IconSize.DIALOG), false, false, 0);
            
            pkg_name = new Label("<big><b>Application name</b></big>");
            pkg_name.use_markup = true;
            pkg_name.halign = Align.START;
            
            short_description = new Label("<i>Application description</i>");
            short_description.use_markup = true;
            short_description.halign = Align.START;
            
            label_box.pack_start(pkg_name, false, false, 0);
            label_box.pack_start(short_description, false, false, 0);
            
            summary_box.pack_start(label_box, false, false, 0);
            
            // Status box
            Box status_box = new Box(Orientation.HORIZONTAL, 0);
            
            Label status_label = new Label("<i>Installed</i>");
            status_label.margin_right = 2;
            status_label.use_markup = true;
            
            status_box.pack_start(new Image.from_stock(Stock.YES, IconSize.MENU), false, false, 0);
            status_box.pack_start(status_label, false, false, 0);
            
            summary_box.pack_end(status_box, false, false, 0);
            
            // Description
            Box description_box = new Box(Orientation.VERTICAL, 0);
            description_box.border_width = 10;
            description = new Label("Package description");
            description.wrap_mode = Pango.WrapMode.WORD_CHAR;
            description.wrap = true;
            description.halign = Align.CENTER;
            description_box.pack_start(description, false, false, 0);
            
            // Details (version and size)
            Box details_box = new Box(Orientation.VERTICAL, 0);
            details_box.border_width = 5;
            version = new Label("<b>Version:</b> 0.0.1");
            version.use_markup = true;
            version.halign = Align.START;
            size = new Label("<b>Size:</b> 1024 Kb");
            size.use_markup = true;
            size.halign = Align.START;
            id = new Label("<b>Id:</b> package-id");
            id.use_markup = true;
            id.halign = Align.START;
            details_box.pack_start(id, false, false, 0);
            details_box.pack_start(version, false, false, 0);
            details_box.pack_start(size, false, false, 0);
            
            // Adding widgets
            box.pack_start(summary_box, false, false, 0);
            pack_separator();
            box.pack_start(description_box, true, true, 0);
            pack_separator();
            box.pack_start(details_box, false, false, 0);
            
            // Reviews container
            reviews_box = new RoundBox(Orientation.VERTICAL, 0);
            reviews_box.expand = false;
            
            Box container = new Box(Orientation.HORIZONTAL, 3);
            container.border_width = 5;
            
            Label reviews_label = new Label("<big><b>Reviews</b></big>");
            reviews_label.use_markup = true;
            reviews_label.halign = Align.START;
            
            container.pack_start(new Image.from_stock(Stock.HELP, IconSize.DIALOG), false, false, 0);
            container.pack_start(reviews_label, false, false, 0);
            
            reviews_box.pack_start(container, false, false, 0);
            
            pack_start(box, true, true, 0);
            pack_start(reviews_box, false, false, 0);
            
            // Buttons box
            ButtonBox b_container = new ButtonBox(Orientation.HORIZONTAL);
            b_container.layout_style = ButtonBoxStyle.END;
            b_container.homogeneous = false;
            
            Button bt = new Button.with_label("Check for reviews");
            
            b_container.pack_start(bt, false, false, 0);
            b_container.pack_start(new Button.with_label("Install"), false, false, 0);
            b_container.foreach((child) => {
                b_container.set_child_non_homogeneous(child, true);
            });
            
            pack_start(b_container, false, false, 0);
        }
    }
}
