
using PackageKit;
using SQLHeavy;
using Menu;

class AppInstallPackage {
    public AppInstallDirectory directory { get; private set; }
    public string name { get; private set; }
    public string icon { get; private set; }
    public string summary { get; private set; }
    public string id { get; private set; }
    
    public AppInstallPackage (AppInstallDirectory directory, string name, string icon, string summary, string id) {
        this.directory = directory;
        this.name = name;
        this.icon = icon;
        this.summary = summary;
        this.id = id;
    }
}

class AppInstallDirectory {
    public string name { get; private set; }
    public string directory { get; private set; }
    
    public AppInstallDirectory (string name, string directory) {
        this.name = name;
        this.directory = directory;
    }
}

class DbBuilder {
    private Dir root;
    private AppInstallDirectory tmp_dir;
    private Transaction trans;
    private KeyFile keys;
    private string desktop_dir;
    private Gee.HashMap<string, AppInstallPackage> packages;
    
    public void add_pkg (string id, Transaction trs) {
        AppInstallPackage app_pkg = packages[id];
        if (app_pkg == null) { return; }
        GLib.debug ("Inserting %s in %s\n", id, app_pkg.directory.name);
        trs.execute_insert("INSERT INTO '%s' VALUES (:id, :name, :description, :icon);".printf(app_pkg.directory.name),
                                   ":id", typeof (string), app_pkg.id,
                                   ":name", typeof (string), app_pkg.name,
                                   ":description", typeof (string), app_pkg.summary,
                                   ":icon", typeof (string), app_pkg.icon);
    }
    
    private bool get_if_included (MenuDir dir, string[] categories) {
        foreach (string in_dir in dir.excluded) {
            if (in_dir in categories) {
                return false;
            }
        }
        foreach (string in_dir in dir.included) {
            if (in_dir in categories) {
                return true;
            }
        }
        return false;
    }
    
    private string get_safe_key (string val) {
        try {
            return keys.get_string ("Desktop Entry", val);
        } catch (GLib.Error e) {
            GLib.debug ("Error retrieving key %s: %s\n", val, e.message);
        }
        return "";
    }
    
    private void get_apps_from_dir (MenuDir dir) {
        root.rewind();
        tmp_dir = new AppInstallDirectory (dir.name, dir.directory);
        string name;
        while ((name = root.read_name ()) != null) {
        if (! name.has_suffix (".desktop")) { continue; }
            try {
                keys.load_from_file (desktop_dir+name, 0);
                if (keys.has_key("Desktop Entry", "Categories") == false) { continue; }
                
                string[] categories = keys.get_string_list ("Desktop Entry", "Categories");
                if (get_if_included (dir, categories)) {
                    AppInstallPackage pkg = new AppInstallPackage (tmp_dir, get_safe_key ("Name"), get_safe_key ("Icon"), get_safe_key ("Comment"), get_safe_key ("X-AppInstall-Package"));
                    packages.set (pkg.id, pkg);
                }
            } catch {
                GLib.debug ("%s is not a valid .desktop file", name);
            }
        }
    }
    
    public DbBuilder (Database db) {
        packages = new Gee.HashMap<string, AppInstallPackage> ();
        
        desktop_dir = "/usr/share/app-install/desktop/";
        root = Dir.open (desktop_dir, 0);
        keys = new KeyFile ();
        
        MenuParser parser = new MenuParser ("/usr/share/app-install/desktop/applications.menu");
        
        trans = db.begin_transaction ();
        
        stdout.printf ("Collecting app-install datas...\n");
        
        foreach (unowned MenuDir item in parser.parse ()) {
            if (item.level == 1) { // We just want the Applications childs
                trans.execute ("CREATE TABLE '%s' (id, name, summary, icon);".printf (item.name));
                get_apps_from_dir (item);
            }
        }
    
        try {
            trans.commit();
        } catch (SQLHeavy.Error e) {
            GLib.error ("Error: %s\n", e.message);
        }
    }
}

Database db;
Client client;
DbBuilder dbbuilder;
Transaction trans;
string[] ids;

void pkg_received (Package pkg) {
    ids += pkg.get_name ();
}

// FIXME: Awful OOP code
// FIXME: doesn't allow same packages in multiple categories

int main () {
    ids = {};
    if (Environment.get_user_name () != "root") {
        GLib.error ("This program needs to be run as root!\n");
    }
    
    FileUtils.remove ("/var/cache/lsc-vala.db");
    db = new Database ("/var/cache/lsc-vala.db");
    
    dbbuilder = new DbBuilder (db);
    
    client = new Client ();
    trans = db.begin_transaction ();
    
    stdout.printf ("Querying PackageKit...\n");
    
    try {
        Bitfield field;
        field = Filter.bitfield_from_string ("arch");
        Results result = client.get_packages(field, null, null);
        result.get_package_array().foreach ((Func) pkg_received);
    } catch (GLib.Error e) {
        GLib.error ("Error: %s\n", e.message);
    }
    
    stdout.printf ("Building database...\n");
    
    foreach (string id in ids) {
       dbbuilder.add_pkg (id, trans);
    }
    
    try {
        trans.commit();
    } catch (SQLHeavy.Error e) {
        GLib.error ("Error: %s\n", e.message);
    }
    
    stdout.printf ("Done, no error reported\n");
    
    return 0;
}
