
namespace Menu {
    class MenuDir : Object {
        public string name { get; set; }
        public string directory { get; set; }
        public string[] included;
        public string[] excluded;
        public int level;
        
        public MenuDir () {
            included = {};
            excluded = {};
        }
    }
    
    class MenuParser {
        private const MarkupParser parser = {
            opening_item,// when an element opens
            closing_item,  // when an element closes
            get_text, // when text is found
            null, // when comments are found
            null  // when errors occur
        };
        
        private string menu_file;
        private MarkupParseContext context;
        private List<MenuDir> dirlist;
        private MenuDir dir;
        private int level = 0;
        private string last_item;
        private bool include = true;
        
        public unowned List<MenuDir> parse () {
            string file;
            FileUtils.get_contents (menu_file, out file, null);
            context.parse (file, file.length);
            
            return dirlist;
        }
        
        public MenuParser (string menu_file) {
            context = new MarkupParseContext(
                parser, // the structure with the callbacks
                0,      // MarkupParseFlags
                this,   // extra argument for the callbacks, methods in this case
                null // when the parsing ends
            );
            
            this.menu_file = menu_file;
        }
        
        void opening_item (MarkupParseContext context, string name, string[] attr, string[] vals) throws MarkupError {
            last_item = name;
            switch (name) {
                case "Menu":
                    dir = new MenuDir();
                    dirlist.append(dir);
                    dir.level = level;
                    level ++;
                    break;
                case "Not":
                    include = false;
                    break;
            }
        }
        
        void closing_item (MarkupParseContext context, string name) throws MarkupError {
            switch (name) {
                case "Menu":
                    level --;
                    break;
                case "Not":
                    include = true;
                    break;
            }
        }
        
        private bool check_whitespaces (string str) {
            return (str.strip() == "" ? true : false);
        }
        
        void get_text (MarkupParseContext context, string text, size_t text_len) throws MarkupError {
            if (check_whitespaces (text)) { return; }
            switch (last_item) {
                case "Name":
                    dir.name = text;
                    break;
                case "Directory":
                    dir.directory = text;
                    break;
                case "Category":
                    if (include) {
                        dir.included += text;
                    } else {
                        dir.excluded += text;
                    }
                    break;
            }
        }
    }
}
